/**
 * Created by irazer on 16/03/17.
 */
"use strict";
var authClient_1 = require("./authClient");
var nock = require("nock");
describe('node-onedrive', function () {
    var BASE_URI = 'https://login.microsoftonline.com/common/oauth2/v2.0';
    var SCOPES = 'offline_mode%20files.read';
    var REDIRECT_URI = 'http%3A%2F%2Flocalhost%2Fcallback';
    beforeEach(function () {
        nock.disableNetConnect();
    });
    it('should return a proper authentication uri', function () {
        var expectedUri = BASE_URI + "/authorize?client_id=1234&redirect_uri=" + REDIRECT_URI + "&response_type=code&scope=" + SCOPES;
        var scopes = [
            'offline_mode',
            'files.read'
        ];
        var client = new authClient_1.AuthClient('1234', 'test', 'http://localhost/callback');
        var result = client.getAuthenticationUrl(scopes);
        expect(result).toBe(expectedUri);
    });
    it('should throw an error saying the scope is missing', function () {
        var client = new authClient_1.AuthClient('1234', 'test', 'http://localhost/callback');
        expect(function () {
            client.getAuthenticationUrl(null);
        }).toThrow(new Error('Scopes must be provided'));
    });
    it('should throw an error saying the URI is missing', function () {
        var scopes = [
            'offline_mode',
            'files.read'
        ];
        var client = new authClient_1.AuthClient('1234', 'test', '');
        expect(function () {
            client.getAuthenticationUrl(scopes);
        }).toThrow(new Error('The redirect URI must be provided'));
        client = new authClient_1.AuthClient('1234', 'test', null);
        expect(function () {
            client.getAuthenticationUrl(scopes);
        }).toThrow(new Error('The redirect URI must be provided'));
    });
    it('should throw an error saying the client id is missing', function () {
        var scopes = [
            'offline_mode',
            'files.read'
        ];
        var client = new authClient_1.AuthClient('', 'test', 'http://localhost/callback');
        expect(function () {
            client.getAuthenticationUrl(scopes);
        }).toThrow(new Error('A client id is required'));
        client = new authClient_1.AuthClient(null, 'test', 'http://localhost/callback');
        expect(function () {
            client.getAuthenticationUrl(scopes);
        }).toThrow(new Error('A client id is required'));
    });
    it('should respond with the token', function (done) {
        nock(BASE_URI, {
            reqheaders: {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        })
            .post('/token', {
            client_id: '1234',
            redirect_uri: 'http://localhost/callback',
            client_secret: 'test secret',
            code: '4321',
            grant_type: 'authorization_code'
        })
            .reply(200, {
            token: 'test token'
        }, {
            'Content-Type': 'application/json'
        });
        var client = new authClient_1.AuthClient('1234', 'test secret', 'http://localhost/callback');
        client.getToken('4321', function (err, res) {
            expect(err).toBeNull();
            expect(res.token).toBe('test token');
            nock.cleanAll();
            done();
        });
    });
    it('should respond with an error for the token', function (done) {
        nock(BASE_URI, {
            reqheaders: {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        })
            .post('/token', {
            client_id: '1234',
            redirect_uri: 'http://localhost/callback',
            client_secret: 'test secret',
            code: '4321',
            grant_type: 'authorization_code'
        })
            .reply(400, 'test error');
        var client = new authClient_1.AuthClient('1234', 'test secret', 'http://localhost/callback');
        client.getToken('4321', function (err, res) {
            expect(err).toBeNull();
            expect(res).toBe('test error');
            nock.cleanAll();
            done();
        });
    });
    it('should respond with the refresh token', function (done) {
        nock(BASE_URI, {
            reqheaders: {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        })
            .post('/token', {
            client_id: '1234',
            redirect_uri: 'http://localhost/callback',
            client_secret: 'test secret',
            refresh_token: '4321',
            grant_type: 'refresh_token'
        })
            .reply(200, {
            token: 'test token'
        }, {
            'Content-Type': 'application/json'
        });
        var client = new authClient_1.AuthClient('1234', 'test secret', 'http://localhost/callback');
        client.refreshToken('4321', function (err, res) {
            expect(err).toBeNull();
            expect(res.token).toBe('test token');
            nock.cleanAll();
            done();
        });
    });
    it('should respond with an error for the refresh token', function (done) {
        nock(BASE_URI, {
            reqheaders: {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        })
            .post('/token', {
            client_id: '1234',
            redirect_uri: 'http://localhost/callback',
            client_secret: 'test secret',
            refresh_token: '4321',
            grant_type: 'refresh_token'
        })
            .reply(400, 'test error');
        var client = new authClient_1.AuthClient('1234', 'test secret', 'http://localhost/callback');
        client.refreshToken('4321', function (err, res) {
            expect(err).toBeNull();
            expect(res).toBe('test error');
            nock.cleanAll();
            done();
        });
    });
});
//# sourceMappingURL=authClient.spec.js.map