/**
 * Created by irazer on 16/03/17.
 */

import { AuthClient } from './authClient';
import * as nock from 'nock';

describe('node-onedrive', () => {
  const BASE_URI = 'https://login.microsoftonline.com/common/oauth2/v2.0';
  const SCOPES = 'offline_mode%20files.read';
  const REDIRECT_URI = 'http%3A%2F%2Flocalhost%2Fcallback';

  beforeEach(() => {
    nock.disableNetConnect();
  });

  it('should return a proper authentication uri', () => {
    let expectedUri = `${BASE_URI}/authorize?client_id=1234&redirect_uri=${REDIRECT_URI}&response_type=code&scope=${SCOPES}`;
    let scopes = [
      'offline_mode',
      'files.read'
    ];

    let client = new AuthClient('1234', 'test', 'http://localhost/callback');
    let result = client.getAuthenticationUrl(scopes);

    expect(result).toBe(expectedUri);
  });

  it('should throw an error saying the scope is missing', () => {
    let client = new AuthClient('1234', 'test', 'http://localhost/callback');

    expect(() => {
      client.getAuthenticationUrl(null);
    }).toThrow(new Error('Scopes must be provided'));
  });

  it('should throw an error saying the URI is missing', () => {
    let scopes = [
      'offline_mode',
      'files.read'
    ];
    let client = new AuthClient('1234', 'test', '');

    expect(() => {
      client.getAuthenticationUrl(scopes);
    }).toThrow(new Error('The redirect URI must be provided'));

    client = new AuthClient('1234', 'test', null);

    expect(() => {
      client.getAuthenticationUrl(scopes);
    }).toThrow(new Error('The redirect URI must be provided'));
  });

  it('should throw an error saying the client id is missing', () => {
    let scopes = [
      'offline_mode',
      'files.read'
    ];
    let client = new AuthClient('', 'test', 'http://localhost/callback');

    expect(() => {
      client.getAuthenticationUrl(scopes);
    }).toThrow(new Error('A client id is required'));

    client = new AuthClient(null, 'test', 'http://localhost/callback');

    expect(() => {
      client.getAuthenticationUrl(scopes);
    }).toThrow(new Error('A client id is required'));
  });

  it('should respond with the token', (done) => {
    nock(BASE_URI, {
      reqheaders: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    })
      .post('/token', {
        client_id: '1234',
        redirect_uri: 'http://localhost/callback',
        client_secret: 'test secret',
        code: '4321',
        grant_type: 'authorization_code'
      })
      .reply(200, {
        token: 'test token'
      }, {
        'Content-Type': 'application/json'
      });

    let client = new AuthClient('1234', 'test secret', 'http://localhost/callback');

    client.getToken('4321', (err, res) => {
      expect(err).toBeNull();
      expect(res.token).toBe('test token');

      nock.cleanAll();
      done();
    });
  });
  it('should respond with an error for the token', (done) => {
    nock(BASE_URI, {
      reqheaders: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    })
      .post('/token', {
        client_id: '1234',
        redirect_uri: 'http://localhost/callback',
        client_secret: 'test secret',
        code: '4321',
        grant_type: 'authorization_code'
      })
      .reply(400, 'test error');

    let client = new AuthClient('1234', 'test secret', 'http://localhost/callback');

    client.getToken('4321', (err, res) => {
      expect(err).toBeNull();
      expect(res).toBe('test error');

      nock.cleanAll();
      done();
    });
  });

  it('should respond with the refresh token', (done) => {
    nock(BASE_URI, {
      reqheaders: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    })
      .post('/token', {
        client_id: '1234',
        redirect_uri: 'http://localhost/callback',
        client_secret: 'test secret',
        refresh_token: '4321',
        grant_type: 'refresh_token'
      })
      .reply(200, {
        token: 'test token'
      }, {
        'Content-Type': 'application/json'
      });

    let client = new AuthClient('1234', 'test secret', 'http://localhost/callback');

    client.refreshToken('4321', (err, res) => {
      expect(err).toBeNull();
      expect(res.token).toBe('test token');

      nock.cleanAll();
      done();
    });
  });

  it('should respond with an error for the refresh token', (done) => {
    nock(BASE_URI, {
      reqheaders: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    })
      .post('/token', {
        client_id: '1234',
        redirect_uri: 'http://localhost/callback',
        client_secret: 'test secret',
        refresh_token: '4321',
        grant_type: 'refresh_token'
      })
      .reply(400, 'test error');

    let client = new AuthClient('1234', 'test secret', 'http://localhost/callback');

    client.refreshToken('4321', (err, res) => {
      expect(err).toBeNull();
      expect(res).toBe('test error');

      nock.cleanAll();
      done();
    });
  });
});
